<?php
/*
Plugin Name: Shopone - Landing Page Builder
Plugin URI: https://shopone.io
Description: Connect to shopone.io builder Landing Page
Author: Shopone
Author URI: http://shopone.io
Version: 1.1
*/
ini_set('display_errors', 0);
ini_set('display_startup_errors', 0);
error_reporting(E_ALL);

if (! class_exists("PageTemplater")) {
  require plugin_dir_path(__FILE__) . 'add-template.php';
}

if (isset($_SERVER['HTTP_ORIGIN'])) {
  header("Access-Control-Allow-Origin: {$_SERVER['HTTP_ORIGIN']}");
  header('Access-Control-Allow-Credentials: true');
  header('Access-Control-Max-Age: 86400'); // cache for 1 day
}

if ($_SERVER['REQUEST_METHOD'] == 'OPTIONS') {
  if (isset($_SERVER['HTTP_ACCESS_CONTROL_REQUEST_METHOD'])) {
    header("Access-Control-Allow-Methods: GET, POST, OPTIONS");
  }
  if (isset($_SERVER['HTTP_ACCESS_CONTROL_REQUEST_HEADERS'])) {
    header("Access-Control-Allow-Headers: {$_SERVER['HTTP_ACCESS_CONTROL_REQUEST_HEADERS']}");
  }
  exit(0);
}

if (! defined('ABSPATH')) {
  exit;
}



function getRandomUserAgent()
{
  $userAgents = array(
    "User-Agent: Mozilla/5.0 (X11; Ubuntu; Linux i686; rv:28.0) Gecko/20100101 Firefox/28.0"
  );
  $random = rand(0, count($userAgents) - 1);
  return $userAgents[$random];
}

function curlPost($url, $data, $apiKey = '')
{
  try {
    $options = array(
      'http' => array(
        'header'  => "Content-Type: application/json\r\n",
        'method'  => 'POST',
        'content' => json_encode($data)
      ),
      "ssl" => array(
        "verify_peer" => false,
        "verify_peer_name" => false
      )
    );

    $context  = stream_context_create($options);
    $result = file_get_contents($url, false, $context);
    // if ($result === FALSE) { echo 1;exit;}
    $resultData = json_decode($result, true);
  } catch (Exception $ex) {
    var_dump($ex);
    exit;
    return false;
  }
}

function curlPost2($url, $data, $apiKey = '')
{
  $data_string = json_encode($data);
  $curl = curl_init($url);
  curl_setopt($curl, CURLOPT_CUSTOMREQUEST, "POST");
  curl_setopt($curl, CURLOPT_POSTFIELDS, $data_string);
  curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
  curl_setopt($curl, CURLOPT_USERAGENT, getRandomUserAgent());
  curl_setopt($curl, CURLOPT_HTTPHEADER, array(
    'Content-Type: application/json',
    'Content-Length: ' . strlen($data_string),
    'Api-Key:' . $apiKey
  ));

  $result = curl_exec($curl);
  curl_close($curl);
  return $result;
}

if (! class_exists('Shopone')) {

  class Shopone
  {
    protected static $_instance = null;

    protected $_notices = array();

    public static function instance()
    {
      if (is_null(self::$_instance)) {
        self::$_instance = new self();
      }

      return self::$_instance;
    }

    public function __construct()
    {
      try {
        _mkdir();
      } catch (Exception $ex) {
      }
      add_action('init', array($this, 'init_endpoint'));
      add_action('admin_init', array($this, 'check_environment'));
      add_action('admin_notices', array($this, 'admin_notices'), 15);
      add_action('admin_menu', array($this, 'add_shopone_menu_item'));
      add_action('wp_ajax_shopone_save_config', array($this, 'save_config'));
      add_action('wp_ajax_shopone_publish_lp', array($this, 'publish_lp'));

       register_activation_hook(__FILE__, array($this, 'activation_process'));
       register_deactivation_hook(__FILE__, array($this, 'deactivation_process'));
    }

     public function activation_process()
     {
      $this->init_endpoint();
      flush_rewrite_rules();
     }

    public function deactivation_process()
     {
      flush_rewrite_rules();
     }

    /* add hook and do action */
    public function init_endpoint()
    {
      add_filter('query_vars', array($this, 'add_query_vars'), 0);
      add_action('parse_request', array($this, 'sniff_requests'));
      add_rewrite_rule('^shopone/api', 'index.php?shopone_api=1', 'top');
    }

    public function add_query_vars($vars)
    {
      $vars[] = 'shopone_api';

      return $vars;
    }

    public function sniff_requests()
    {
      global $wp;
      $config = get_option('shopone_config', '');
      $params      = filter_input_array(INPUT_POST);
      $isShopone = isset($wp->query_vars['shopone_api']) ? $wp->query_vars['shopone_api'] : null;
      $actProduct = isset($_GET['action']) ? $_GET['action'] : null;

      // Action for Intergrated products & orders.

      if (! is_null($isShopone) && $isShopone === "1") {
        $api_key     = isset($params['token']) ? sanitize_text_field($params['token']) : null;
        $action      = isset($params['action']) ? sanitize_text_field($params['action']) : null;
        $url         = isset($params['url']) ? sanitize_text_field($params['url']) : null;
        $title       = isset($params['title']) ? sanitize_text_field($params['title']) : null;
        $html        = isset($params['html']) ? $params['html'] : '';
        $type       = isset($params['type']) ? sanitize_text_field($params['type']) : null;
        $idp       = isset($params['idp']) ? sanitize_text_field($params['idp']) : 0;

        // Check key for action products & orders
        if (isset($_GET['ldp_token'])) {
          $api_key = @$_GET['ldp_token'];
        }

        if ($api_key !== $config['api_key']) {
          wp_send_json(array(
            'code'    => 2
          ));
        }

        // Action for Intergrated products & orders.
        if ($actProduct !== null) {
          switch ($actProduct) {
            case 'order_create':
              $this->createOrder();
              break;
          }
        }

        // Action for publish Landing Page
        switch ($action) {
          case 'create':
            if ($this->get_id_by_slug($url)) {
              wp_send_json(array(
                'code'    => 2,
                'message'  => 'url da ton tai'
              ));
            }
            kses_remove_filters();
            if ($type == null) {
              $post_type = 'page';
            } else {
              $post_type = $type;
            }

            $id = wp_insert_post(
              array(
                'post_title' => $title,
                'post_name' => $url,
                'post_type' => $post_type,
                'post_content' => trim($html),
                'post_status' => 'publish',
                'filter' => true,
                'page_template'  => 'null-template.php'
              )
            );

            if ($id) {
              updateSource($url, $html);
              wp_send_json(array(
                'code'    => 1,
                'url'    => get_permalink($id),
                'idp' => $id,
                'message' => 'Tao moi thanh cong'
              ));
            } else {
              wp_send_json(array(
                'code'    => 0,
                'message' => __('Create failed, please try again.')
              ));
            }
            break;

          case 'update':

            if($idp < 1){
              wp_send_json(array(
                'code'    => 4,
                'message' =>'Thieu idp khi cap nhat'
              ));
            }else{
              $post = get_post($idp);
              $slug = $post->post_name;    
  
              if($post == NULL){
                wp_send_json(array(
                  'code'    => 5,
                  'message' =>'Idp khong ton tai'
                ));
                
              }
            }

            if($slug != $url){
              if ($this->get_id_by_slug($url)) {
                wp_send_json(array(
                  'code'    => 2,
                  'message'  => 'url da ton tai'
                ));
              }
              stopLandingPage($slug);
            }
            
           
            $post = array(
              'ID' => $idp,
              'post_title' => $title,
              'post_name' => $url,
              'post_content' => trim($html),
            );

            kses_remove_filters();

            if (wp_update_post($post)) {
              updateSource($url, $html);
              wp_send_json(array(
                'code'    => 1,
                'url'    => get_permalink($idp)
              ));
            } else {
              wp_send_json(array(
                'code'    => 0,
                'message' => 'Cap nhat loi'
              ));
            }
            break;

          case 'delete':
            if (! $this->get_id_by_slug($url)) {
              wp_send_json(array(
                'code'    => 0,
                'message' => __('URL does not exist')
              ));
            } else {
              $id = $this->get_id_by_slug($url);
              $result = wp_delete_post($id);
              try {
                stopLandingPage($url);
              } catch (Exception $ex) {
                wp_send_json(array(
                  'code'    => 0,
                  'message' => __('Delete failed, please try again.')
                ));
              }

              if ($result) {
                wp_send_json(array(
                  'code'    => 1
                ));
              } else {
                wp_send_json(array(
                  'code'    => 0,
                  'message' => __('Delete failed, please try again.')
                ));
              }
            }
            break;
          case 'checkurl':
            if (! $this->get_id_by_slug($url)) {
              wp_send_json(array(
                'code'    => 1
              ));
            } else {
              wp_send_json(array(
                'code'    => 0
              ));
            }
            break;
          case 'checktoken':
            if ($api_key === $config['api_key']) {
              wp_send_json(array(
                'code'    => 1
              ));
            }
            break;
          default:
            wp_send_json(array(
              'code'    => 3,
              'message' => __('Shopone action is not set or incorrect.')
            ));
            break;
        }
      }
    }


    protected function get_id_by_slug($page_slug)
    {
      $page = get_page_by_path($page_slug, 'OBJECT', ['post', 'page', 'product', 'property']);
      if ($page) {
        return $page->ID;
      } else {
        return null;
      }
    }

    public function get_option($id, $default = '')
    {
      $options = get_option('shopone_config', array());
      if (isset($options[$id]) && $options[$id] != '') {
        return $options[$id];
      } else {
        return $default;
      }
    }

    /* Add menu and option */
    public function check_environment()
    {
      if (is_plugin_active(plugin_basename(__FILE__))) {
        if (! function_exists('curl_init')) {
          $this->add_admin_notice('curl_not_exist', 'error', __('Shopone requires cURL to be installed.', 'Shopone'));
        }
      }
    }

    public function add_admin_notice($slug, $class, $message)
    {
      $this->_notices[$slug] = array(
        'class'   => $class,
        'message' => $message,
      );
    }

    public function admin_notices()
    {
      foreach ($this->_notices as $notice_key => $notice) {
        echo "<div class='" . esc_attr($notice['class']) . "'><p>";
        echo wp_kses($notice['message'], array('a' => array('href' => array())));
        echo '</p></div>';
      }
    }

    public function add_shopone_menu_item()
    {
      add_menu_page(__("Shopone Page"), __("Shopone Page"), "manage_options", "shopone-config", array(
        $this,
        'shopone_settings_page'
      ), null, 30);
    }

    public function shopone_settings_page()
    {
      if (! empty($this->_notices)) {
?>
        <div>Please install cURL to use ShopOne plugin</div>
      <?php
      } else {
      ?>

        <div class="wrap">
          <h2 class="title">Shopone Landing Page Builder</h2><br>


          <div>

            <form id="shopone_config" class="shoponeui-panel">
              <!-- <h3><strong>Config API Key</strong></h3> -->
              <table class="form-table">
              <tr>
                  <th scope="row"  style="width:100px">
                    <label for="webiste_shopone">Website URL</label>
                  </th>
                  <td>
                    <input onClick="this.select();" readonly="readonly" name="webiste_shopone" id="webiste_shopone" type="text" class="regular-text shoponeui input" value="<?php echo get_home_url(); ?>">
                  </td>
                </tr>
                <tr>
                  <th scope="row" style="width:100px">
                    <label for="api_key">API KEY</label>
                  </th>
                  <td>
                    <?php
                    $config = get_option('shopone_config', array());

                    if (!isset($config['api_key']) || trim($config['api_key']) == '') {
                      $config['api_key'] = $this->generateRandomString(32);
                      update_option('shopone_config', $config);
                    }

                    ?>
                    <input onClick="this.select();" readonly="readonly" name="api_key" id="api_key" type="text" class="regular-text shoponeui input"
                      value="<?php echo $this->get_option('api_key', ''); ?>">
                    <button type="button" id="shopone_new_api" class="shoponeui button primary">NEW API KEY</button>
                  </td>
                </tr>
           
              </table>

              <div class="submit">
                <button class="button button-primary shoponeui button primary" id="shopone_save_option" type="button">Save Changes
                </button>
              </div>
            </form>

          </div>
          <script>
            (function($) {
              function generateRandomString(length = 10) {
                var text = "";
                var possible = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789";
                for (var i = 0; i < length; i++)
                  text += possible.charAt(Math.floor(Math.random() * possible.length));
                return text;
              }

              $(document).ready(function() {
                $('#shopone_save_option').on('click', function(event) {
                  var data = JSON.stringify($('#shopone_config').serializeArray());
                  $.ajax({
                    url: ajaxurl,
                    type: 'POST',
                    data: {
                      action: 'shopone_save_config',
                      data: data
                    },
                    success: function(response) {
                      alert('Save Success');
                    }
                  });
                  event.preventDefault();
                });
                $("#shopone_new_api").click(function() {
                  var api = generateRandomString(32);
                  $("#shopone_config #api_key").val(api);
                });

                $('#shopone_publish').on('click', function(event) {
                  event.preventDefault();
                  $('#shopone-message').removeClass('lp-hide');
                  $('#shopone-message').empty().text('Processing...');
                  var shopOneKey = $('#shopone_key').val();
                  if (shopOneKey == '') {
                    alert('Please enter your ShopOne Key!');
                    return false;
                  }

                  $.ajax({
                    url: ajaxurl,
                    type: 'POST',
                    data: {
                      action: 'shopone_publish_lp',
                      shopone_key: shopOneKey
                    },
                    success: function(res) {
                      $('#shopone-message').empty().html(res.message);
                    }
                  });
                  event.preventDefault();

                });

        

              });
            })(jQuery);
          </script>
        </div>
<?php
      }
    }

    public function save_config()
    {
      $data   = sanitize_text_field($_POST['data']);
      $data   = json_decode(stripslashes($data));
      $option = array();

      foreach ($data as $key => $value) {
        $option[$value->name] = $value->value;
      }
      update_option('shopone_config', $option);
      die;
    }


    public function generateRandomString($length = 10)
    {
      $characters = '0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ';
      $charactersLength = strlen($characters);
      $randomString = '';
      for ($i = 0; $i < $length; $i++) {
        $randomString .= $characters[rand(0, $charactersLength - 1)];
      }
      return $randomString;
    }
  }

  function Shopone()
  {
    return Shopone::instance();
  }

  Shopone();
}

function get_web_page($request, $post = 0)
{
  $data = array('message' => '', 'content' => '');

  if (function_exists('curl_exec')) {
    $ch = curl_init();
    if ($post == 1) {
      curl_setopt($ch, CURLOPT_POST, 1);
    }
    curl_setopt($ch, CURLOPT_USERAGENT, getRandomUserAgent());
    curl_setopt($ch, CURLOPT_RETURNTRANSFER, 5);
    curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
    curl_setopt($ch, CURLOPT_URL, $request);
    curl_setopt($ch, CURLOPT_TIMEOUT, 60);
    $response = curl_exec($ch);
    if (!$response) {
      $data['message'] = 'cURL Error Number ' . curl_errno($ch) . ' : ' . curl_error($ch);
    } else {
      $data['content'] = $response;
    }
    curl_close($ch);
  }

  return $response;
}

function _mkdir()
{
  $sourceFolder = WP_CONTENT_DIR . '/shopone/';
  if (!file_exists($sourceFolder) && !is_dir($sourceFolder)) {
    @mkdir($sourceFolder, 0777);
  }
}

function stopLandingPage($url)
{
  try {
    $sourceFolder = WP_CONTENT_DIR . '/shopone/';
    $path = $sourceFolder . '/' . $url . '.html';
    if (file_exists($path)) {
      @unlink($path);
    }
  } catch (Exception $ex) {
  }
}

function updateSource($url, $html)
{
  try {
    $sourceFolder = WP_CONTENT_DIR . '/shopone/';
    $path = $sourceFolder . '/' . $url . '.html';
    @file_put_contents($path, $html);
  } catch (Exception $ex) {
  }
}
?>